<?php

namespace Drupal\config_enforce_devel\Form;

use Drupal\config_enforce\ConfigEnforce;
use Drupal\config_enforce\Form\FormHelperTrait;
use Drupal\config_enforce_devel\ConfigEnforceDevel;
use Drupal\Core\Config\FileStorage;
use Drupal\Core\Config\InstallStorage;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Defines a settings form for the Config Enforce Devel module.
 */
trait DevelFormHelperTrait {

  use FormHelperTrait;

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->setSharedFormProperties($form, $form_state);

    $this->saveConfig();

    return parent::submitForm($this->form(), $this->formState());
  }

  /**
   * Add fields for a single config object.
   */
  protected function addEnforcedConfigItem($config) {
    parent::addEnforcedConfigItem($config);

    $enforced = ConfigEnforce::isEnforced($config);
    $enabled_field_name = 'config_enforce_enabled-' . $config;
    $this->form()['config_enforce'][$config]['config_enforce_enabled'] = [
      '#type' => 'checkbox',
      '#title' => 'Enforce config',
      '#description' => "Enable enforcement of the config from this form.",
      '#default_value' => $enforced,
      '#attributes' => [
        'config-enforce-enabled' =>  $enabled_field_name
      ],
    ];

    $states = [
      'invisible' => [
        ':input[config-enforce-enabled="' . $enabled_field_name . '"]' => ['checked' => FALSE],
      ],
    ];

    $this->form()['config_enforce'][$config]['config_enforce_module'] = $this->getTargetModuleField();
    $this->form()['config_enforce'][$config]['config_enforce_module']['#states'] = $states;
    if ($enforced) {
      $module = ConfigEnforce::getModule($config);
      $this->form()['config_enforce'][$config]['config_enforce_module']['#title'] .= ' (<em>' . $module . '</em>)';
      $this->form()['config_enforce'][$config]['config_enforce_module']['#default_value'] = $module;
    }

    $this->form()['config_enforce'][$config]['config_enforce_directory'] = $this->getConfigDirectoryField();
    $this->form()['config_enforce'][$config]['config_enforce_directory']['#states'] = $states;
    if ($enforced) {
      $directory = ConfigEnforce::getDirectory($config);
      $this->form()['config_enforce'][$config]['config_enforce_directory']['#title'] .= ' (<em>' . $directory . '</em>)';
      $this->form()['config_enforce'][$config]['config_enforce_directory']['#default_value'] = $directory;
    }

    if ($enforced) {
      $this->form()['config_enforce'][$config]['config_enforce_path']['#states'] = $states;
    }

    $this->form()['config_enforce'][$config]['config_enforce_level'] = $this->getEnforcementLevelField();
    $this->form()['config_enforce'][$config]['config_enforce_level']['#states'] = $states;
    if ($enforced) {
      $this->form()['config_enforce'][$config]['config_enforce_level']['#title'] .= ' (<em>' . ConfigEnforce::getLevelLabel($config) . '</em>)';
      $this->form()['config_enforce'][$config]['config_enforce_level']['#default_value'] = ConfigEnforce::getLevel($config);
    }
  }

  /**
   * Save our config.
   */
  protected function saveConfig() {
    $this->configEnforce = new ConfigEnforceDevel();
    foreach ($this->getFormConfigs() as $config) {
      $this->configEnforce->setConfig($config);

      if (!$this->getEnforced($config)) {
        $this->configEnforce->clear();
        continue;
      }

      $this->prepareConfig($config);

      $this->configEnforce->save();
    }
  }

  /**
   * Prepare our config.
   */
  protected function prepareConfig($config) {
    $this->configEnforce
      ->setModule($this->getModule($config))
      ->setDirectory($this->getDirectory($config))
      ->setLevel($this->getLevel($config));
  }

  /**
   * Return a list of modules.
   */
  protected function getAllModules() {
    $modules = [];
    $moduleHandler = \Drupal::moduleHandler();
    foreach ($moduleHandler->getModuleList() as $name => $module) {
      $modules[$name] = $moduleHandler->getName($name);
    }
    asort($modules);
    return $modules;
  }

  /**
   * Return a list of available modules to target for writing config.
   */
  protected function getAvailableModules() {
    $modules = [];
    $available_modules = $this->config('config_enforce_devel.settings')->get('available_modules');
    $available_modules = !is_null($available_modules) ? $available_modules : [];
    foreach ($available_modules as $module) {
      $modules[$module] = $this->getAllModules()[$module];
    }
    return $modules;
  }

  /**
   * Return a human-readable name for a given module.
   */
  protected function getModuleName($module) {
    return $this->getAvailableModules()[$module];
  }

  /**
   * Return a list of directories for writing config.
   */
  protected function getAvailableDirectories() {
    // @TODO Look this up in the storage service or make this a hook.
    return [
      InstallStorage::CONFIG_INSTALL_DIRECTORY => 'Install',
      InstallStorage::CONFIG_OPTIONAL_DIRECTORY => 'Optional',
    ];
  }

  /**
   * Return a human-readable name for a given config directory.
   */
  protected function getDirectoryLabel($directory) {
    return $this->getAvailableDirectories()[$directory];
  }

  /**
   * Return a list of all config object names from given module and directory.
   */
  protected function getModuleConfigNames($module, $directory) {
    $module_config_path = drupal_get_path('module', $module) . '/' . $directory;
    $storage = new FileStorage($module_config_path, StorageInterface::DEFAULT_COLLECTION);
    return $storage->listAll();
  }

  /**
   * Return array of config enforce values from the form.
   */
  protected function getConfigEnforceValuesFromForm() {
    return $this->formState()->getValue('config_enforce');
  }

  /**
   * Return the value of a specific field for a given config object.
   */
  protected function getConfigEnforceValue(string $config, string $field) {
    $values = $this->getConfigEnforceValuesFromForm();
    if ($this->areConfigEnforceValuesWrapped()) {
      // Some forms appear to wrap our settings in an array.
      $values = $values['config_enforce'];
    }
    return $values[$config][$field];
  }

  /**
   * Determine whether config enforce values are wrapped in an extra array.
   */
  protected function areConfigEnforceValuesWrapped() {
    $values = $this->getConfigEnforceValuesFromForm();
    return array_key_exists('config_enforce', $values);
  }

  /**
   * Return the new target module from the form state.
   */
  protected function getModule($config) {
    return $this->getConfigEnforceValue($config, 'config_enforce_module');
  }

  /**
   * Return the new config directory from the form state.
   */
  protected function getDirectory($config) {
    return $this->getConfigEnforceValue($config, 'config_enforce_directory');
  }

  /**
   * Return the new enabled setting from the form state.
   */
  protected function getEnforced($config) {
    return $this->getConfigEnforceValue($config, 'config_enforce_enabled');
  }

  /**
   * Return the new enforcement level from the form state.
   */
  protected function getLevel($config) {
    return $this->getConfigEnforceValue($config, 'config_enforce_level');
  }

  /**
   * Return the basic definition for a target module field.
   */
  protected function getTargetModuleField() {
    return [
      '#type' => 'select',
      '#title' => $this->t('Target module'),
      '#description' => "The module to which config should be written.",
      '#options' => $this->getAvailableModules(),
    ];
  }

  /**
   * Return the basic definition for a config directory field.
   */
  protected function getConfigDirectoryField() {
    return [
      '#type' => 'select',
      '#title' => $this->t('Directory'),
      '#description' => "The config directory to which config should be written.",
      '#options' => $this->getAvailableDirectories(),
    ];
  }

  /**
   * Return the basic definition for an enforcement level field.
   */
  protected function getEnforcementLevelField() {
    return [
      '#type' => 'select',
      '#title' => $this->t('Enforcement level'),
      '#description' => "The level of enforcement to apply to config objects.",
      '#options' => ConfigEnforce::getLevels(),
      '#default_value' => ConfigEnforce::CONFIG_ENFORCE_NOSUBMIT,
    ];
  }


}
